package Example;
use strict;
use warnings;
our $VERSION = '0.01';

1;
__END__

=head1 NAME

Example -

=head1 SYNOPSIS

  use Example;

=head1 DESCRIPTION

Example is

=head1 AUTHOR

Kazuhiro Osawa E<lt>yappo {at} shibuya {dot} plE<gt>

=head1 SEE ALSO

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
